<?php
/*
 * This file belongs to the YIT Framework.
 *
 * This source file is subject to the GNU GENERAL PUBLIC LICENSE (GPL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.gnu.org/licenses/gpl-3.0.txt
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
} // Exit if accessed directly

/*
 * @package Yithemes
 * @since  1.0.0
 * @author Yithemes
 */

if ( ! class_exists( 'YITH_PayPal_EC_Response_Payment' ) ) {
	class YITH_PayPal_EC_Response_Payment extends YITH_PayPal_EC_Response {

		/** @var array list of parameters */
		protected $success_payments = array( 'Completed', 'Processed', 'In-Progress' );

		/** @var string */
		protected $prefix = 'PAYMENTINFO_0_';


		/**
		 * Constructor
		 *
		 * Initialize plugin and registers actions and filters to be used
		 *
		 * @since  1.2.0
		 *
		 * @param $response
		 */
		public function __construct( $response, $prefix = 'PAYMENTINFO_0_'  ) {
			parent::__construct( $response );
		}


		/**
		 * Checks if the transaction was approved
		 *
		 * @return bool
		 * @since  1.2.0
		 */
		public function transaction_approved() {
			return in_array( $this->get_response_payment_parameter( 'PAYMENTSTATUS' ), $this->success_payments );
		}

		/**
		 * Returns the name for this
		 * @param $name
		 *
		 * @return string
		 */
		protected function get_response_payment_parameter_name( $name ){
			return $this->prefix. $name;
		}

		/**
		 * Returns a payment parameter
		 *
		 * @param $name
		 *
		 * @return string
		 */
		public function get_response_payment_parameter( $name ) {
			$name = $this->get_response_payment_parameter_name( $name );

			return $this->get_response_parameter( $name );
		}

		/**
		 * Returns the BILLINGAGREEMENTID parameter of the response
		 * after  get_express_checkout_details request
		 * @return int|mixed
		 */
		public function get_payment_status () {
			return isset( $this->response_parameters['BILLINGAGREEMENTID'] ) ? $this->response_parameters['BILLINGAGREEMENTID'] : 0;
		}
	}

}

/**
 * Implements  Class
 *
 * @class   YWSBS_PayPal_EC_Payment_Response_Reference_Transaction
 * @package YITH WooCommerce Subscription
 * @since   1.2.0
 * @author  Yithemes
 */
if ( ! class_exists( 'YITH_PayPal_EC_Response_Payment_Reference_Transaction' ) ) {

	class YITH_PayPal_EC_Response_Payment_Reference_Transaction extends YITH_PayPal_EC_Response_Payment {

		protected $prefix = '';

		/**
		 * Constructor
		 *
		 * Initialize plugin and registers actions and filters to be used
		 *
		 * @since  1.2.0
		 *
		 * @param $response
		 */
		public function __construct( $response ) {
			parent::__construct( $response );
		}


		/*public function has_error() {
			return true;
		}

		public function transaction_approved() {
			return false;
		}

		public function get_errors( $first = false ) {
			$error = array(
				'code'    => '100060',
				'message' => 'Test fail'
			);

			return $error;
		}*/

	}

}

